'''
Created on 10 sep 2012

@author: tomlun
'''
from lib.game import Game
from lib.menu import Menu
from lib.renderer.framedispatcher import FrameDispatcher
from lib.renderer.root import Root
import pyglet
import sys

class App(object):
    '''
    classdocs
    '''
    _views = {}
    
    pyglet.font.add_file("data/Zombielawn.ttf")
    zombie_lawn_font = pyglet.font.load("Zombielawn")

    def __init__(self):
        '''
        Constructor
        '''
        self.root = Root()
        self.window = self.root.initialize("PyWeek15")
        
        FrameDispatcher(self)
        
        menu = Menu(self)
        game = Game(self)
        
        self._views["Menu"] = menu
        self._views["Game"] = game
        
        self.set_view("Menu")
        
    def run(self):
        while not self.window.has_exit:
            self.root.render_one_frame()
            
    def stop(self):
        # TODO: Cleanup
        sys.exit()
    
    def frame_started(self, dt):
        pass
        
    def frame_ended(self, dt):
        for key in [key for key in self._views if self._views[key].active]:
            self._views[key].update(dt)
            
    def set_view(self, view_name):
        view = self._views[view_name]        
        view.active = True
        self.push_view(view)
        
        for key in [key for key in self._views if not key == view_name]:
            view = self._views[key]
            view.active = False
            self.pop_view(view)
            
    def push_view(self, view):
        if view.scene_manager:
            view.scene_manager.active = True
        
        self.window.push_handlers(
                  view.on_mouse_press,
                  view.on_mouse_release,
                  view.on_mouse_motion,
                  view.on_key_press,
                  view.on_key_release)
    
    def pop_view(self, view):
        if view.scene_manager:
            view.scene_manager.active = False
        
        self.window.remove_handlers(
                  view.on_mouse_press,
                  view.on_mouse_release,
                  view.on_mouse_motion,
                  view.on_key_press,
                  view.on_key_release)