'''
Created on 12 sep 2012

@author: tommy.lundgren
'''
from lib.bonuspoint import BonusPoint
from lib.enemy import Enemy
from lib.floattext import FloatText
from lib.fuel import Fuel
from lib.player import Player
from lib.sky import Sky
from lib.view import View
from pyglet.window import key
import lib
import math
import pyglet

class Game(View):
    '''
    classdocs
    '''
    _fuel_spawn = 7
    _bonus_spawn = 10
    _enemy_spawn = 3
    
    _fuel_texture = pyglet.image.load("data/fuel.png")

    def __init__(self, parent):
        '''
        Constructor
        '''
        View.__init__(self, parent)        
        
        self.scene_manager = lib.Root().create_scene_manager("GameSceneManager")                
        self.batch = pyglet.graphics.Batch()
        self.text_batch = pyglet.graphics.Batch()
        self.fuel_batch = pyglet.graphics.Batch()
        self.bonus_batch = pyglet.graphics.Batch()
        self.enemy_batch = pyglet.graphics.Batch()
                
        text_group = pyglet.graphics.OrderedGroup(5)
        
        ''' Game components '''        
        self.sky = Sky(self.scene_manager, self.batch)
        self.player = Player(self.scene_manager)
              
        self.score = 0
        ''' Timers '''
        self.fuel_timer = 3
        self.bonus_timer = 2
        self.enemy_timer = -5
        
        ''' Lists '''
        self.fuel_entities = []
        self.bonus_entities = []
        self.enemy_entities = []
        self.float_texts = []
                
        ''' Setup labels '''
        text_color = (24, 46, 255, 255)        
        self.score_label = pyglet.text.Label(font_name = "Zombielawn",
                                             font_size = 26,
                                             x = 20, y = parent.window.height - 46,
                                             color = text_color,
                                             batch = self.text_batch,
                                             group = text_group)
        
        self.fuel_label = pyglet.text.Label(font_name = "Zombielawn",
                                            font_size = 26,
                                            x = 20, y = parent.window.height - 92,
                                            color = text_color,
                                            batch = self.text_batch,
                                            group = text_group)
        
        self.game_over_label = pyglet.text.Label("GAME OVER",
                                                 font_name = "Zombielawn",
                                                 font_size = 80,
                                                 color = (255, 0, 0, 255),
                                                 group = text_group)
        
        self.game_over_label.x = (parent.window.width / 2) - (self.game_over_label.content_width / 2)
        self.game_over_label.y = (parent.window.height / 2) - (self.game_over_label.content_height / 2)
        
        
        self.btn_batch = pyglet.graphics.Batch()
        self.r_btn = lib.ui.Button("PLAY AGAIN", self.btn_batch)
        self.r_btn.x = (parent.window.width / 2) - (self.r_btn.content_width / 2)
        self.r_btn.y = 250
        self.r_btn.callback = self.reset
        
        ''' Nasty fix '''
        self.scene_manager._add_batch(self.text_batch)
   
    def reset(self):
        self.score = 0
        self.player.fuel = 100
        self.player.node.position = (1024 / 2 - 32, 500)
        self.player.alive = True
        self.player.force_up = 100
        self.player.force_strafe = 0
        
        ''' Timers '''
        self.fuel_timer = 3
        self.bonus_timer = 2
        self.enemy_timer = -5
        
        ''' Lists '''
        for fuel in self.fuel_entities:
            self.scene_manager.remove_scene_node(fuel.node)
            self.fuel_entities.remove(fuel)
        
        for bonus in self.bonus_entities:
            self.scene_manager.remove_scene_node(bonus.node)
            self.bonus_entities.remove(bonus)
        
        for enemy in self.enemy_entities:
            self.scene_manager.remove_scene_node(enemy.node)
            self.enemy_entities.remove(enemy)
         
    def update(self, dt):
        self.sky.update(dt)
        self.player.update(dt)
        self.score_label.text = "SCORE: %i" % math.ceil(self.score) 
        self.fuel_label.text = "FUEL: %i" % math.ceil(self.player.fuel)
        
        
        ''' Start Fuel '''
        if math.ceil(self.fuel_timer) > self._fuel_spawn:
            self.fuel_entities.append(Fuel(self.scene_manager, self.fuel_batch))
            self.fuel_timer = 0
        self.fuel_timer += dt
        
        for fuel in self.fuel_entities:
            if fuel.node.y + fuel.texture.height < 0:
                self.scene_manager.remove_scene_node(fuel.node)
                self.fuel_entities.remove(fuel)
                continue
            else:
                fuel.update(dt)
            
            if fuel.node.collides(self.player.node):
                self.add_float_text("+ FUEL", fuel.node.x, fuel.node.y)
                self.scene_manager.remove_scene_node(fuel.node)
                self.fuel_entities.remove(fuel)
                self.player.fuel = 100 
        ''' End Fuel '''
        
        ''' Start Bonus '''
        if math.ceil(self.bonus_timer) > self._bonus_spawn:
            self.bonus_entities.append(BonusPoint(self.scene_manager, self.bonus_batch))
            self.bonus_timer = 0      
        self.bonus_timer += dt
        
        for bonus in self.bonus_entities:
            if bonus.node.y + bonus.texture.height < 0:
                self.scene_manager.remove_scene_node(bonus.node)
                self.bonus_entities.remove(bonus)
                continue
            else:
                bonus.update(dt)
            
            if bonus.node.collides(self.player.node):
                self.add_float_text("+ 30 SCORE", bonus.node.x, bonus.node.y)
                self.scene_manager.remove_scene_node(bonus.node)
                self.bonus_entities.remove(bonus)
                self.score += 30 
        ''' End Bonus '''
                
        ''' Start Enemies '''
        if math.ceil(self.enemy_timer) > self._enemy_spawn:
            self.enemy_entities.append(Enemy(self.scene_manager, self.enemy_batch))
            self.enemy_timer = 0      
        self.enemy_timer += dt
        
        for enemy in self.enemy_entities:
            if enemy.direction == Enemy.DIR_LEFT:
                if enemy.node.x + enemy.texture2.width < 0:
                    self.scene_manager.remove_scene_node(enemy.node)
                    self.enemy_entities.remove(enemy)
                    continue
            elif enemy.direction == Enemy.DIR_RIGHT:
                if enemy.node.x > self.parent.window.width:
                    self.scene_manager.remove_scene_node(enemy.node)
                    self.enemy_entities.remove(enemy)
                    continue
            
            enemy.update(dt)
            
            if enemy.node.collides(self.player.node):
                ''' Player got hit by enemy, player dead '''
                self.player.alive = False
                self.player.node.position = (-64, 500)
        ''' End Enemies '''
             
        
        for text in self.float_texts:
            text.update(dt)
            if not text.alive:
                self.float_texts.remove(text)
                text.delete()
       
        if self.player.alive:
            self.score += 1 * dt
        else:
            ''' Game is over '''
            self.game_over_label.draw()   
            self.btn_batch.draw()  
    
    def add_float_text(self, text, x, y):
        self.float_texts.append(FloatText(text, x, y, self.text_batch))
    
    def on_key_press(self, symbol, modifiers):        
        if symbol == key.UP:
            self.player.jetpack_enabled = True
            self.player.set_jet_tex()         
        if symbol == key.RIGHT:
            self.player.strafe_right = True
            self.player.direction = 1
            if self.player.jetpack_enabled:
                self.player._set_dir_r_jet_tex()
            else:                
                self.player._set_dir_r_tex()
        elif symbol == key.LEFT:
            self.player.strafe_left = True
            self.player.direction = 0
            if self.player.jetpack_enabled:
                self.player._set_dir_l_jet_tex()
            else:                
                self.player._set_dir_l_tex()
    
    def on_key_release(self, symbol, modifiers):
        if symbol == key.UP:
            self.player.jetpack_enabled = False
            self.player.set_nor_tex()        
        if symbol == key.RIGHT:
            self.player.strafe_right = False
        if symbol == key.LEFT:
            self.player.strafe_left = False
            
    def on_mouse_release(self, x, y, button, modifiers):
        if not self.player.alive:
            if self.r_btn.hit_test(x, y):
                self.r_btn.callback()
            
    def on_mouse_motion(self, x, y, dx, dy): 
        if not self.player.alive:
            if self.r_btn.hit_test(x, y):
                self.r_btn.hover()
            else:
                self.r_btn.reset_color()