'''
Created on 12 sep 2012

@author: tommy.lundgren
'''
import lib
import pyglet

class Player(object):
    '''
    classdocs
    '''
    tex_dir_r = pyglet.image.load("data/player_dir_r.png")
    tex_dir_r_jet = pyglet.image.load("data/player_dir_r_jet.png")
    tex_dir_l = pyglet.image.load("data/player_dir_l.png")
    tex_dir_l_jet = pyglet.image.load("data/player_dir_l_jet.png")
    gravity = 100
    max_force_up = 250
    max_force_strafe = 150

    def __init__(self, scene_manager):
        '''
        Constructor
        '''
        batch = pyglet.graphics.Batch()
                
        self.node = scene_manager.create_scene_node("Player")
        entity = lib.Entity(self.tex_dir_r, batch)        
        self.node.attach(entity)
        self.node.position = (1024 / 2 - 32, 500)
        
        self.fuel = 100
        self.alive = True
        
        ''' Forces ''' 
        self.force_up = 0
        self.force_strafe = 0
        
        self.jetpack_enabled = False
        self.strafe_right = False
        self.strafe_left = False
        self.direction = 1
    
    def set_jet_tex(self):
        if self.direction == 0:
            self._set_dir_l_jet_tex()
        else:
            self._set_dir_r_jet_tex()
    
    def set_nor_tex(self):
        if self.direction == 0:
            self._set_dir_l_tex()
        else:
            self._set_dir_r_tex()
    
    def _set_dir_r_jet_tex(self):
        self.node.entity.image = self.tex_dir_r_jet
    
    def _set_dir_r_tex(self):
        self.node.entity.image = self.tex_dir_r
        
    def _set_dir_l_jet_tex(self):
        self.node.entity.image = self.tex_dir_l_jet
    
    def _set_dir_l_tex(self):
        self.node.entity.image = self.tex_dir_l
    
    def update(self, dt):
        if not self.alive:
            return
        
        self.node.y -= self.gravity * dt
        self.node.y += self.force_up * dt
        self.node.x += self.force_strafe * dt
        
        ''' Up movement '''
        if self.jetpack_enabled and self.fuel > 0:       
            self.fuel -= 20 * dt
            self.gravity = 100
            if self.force_up < self.max_force_up:
                self.force_up += 150 * dt            
        else:
            if self.gravity < 400:
                self.gravity += 100 * dt
            if self.force_up > 0:
                self.force_up -= 100 * dt
        
        ''' Sideways movement ''' 
        if self.strafe_right:
            if self.force_strafe < self.max_force_strafe:
                self.force_strafe += 100 * dt                
        elif self.strafe_left:
            if self.force_strafe > -self.max_force_strafe:
                self.force_strafe -= 100 * dt
        else:
            if self.force_strafe > 0:
                self.force_strafe -= 100 * dt
            else:
                self.force_strafe += 100 * dt
              
        ''' Window border collision '''
        if self.node.x < 0:
            self.node.x = 0
            self.force_strafe = 0
        elif self.node.x + 64 > 1024:
            self.node.x = 1024 - 64
            self.force_strafe = 0
        if self.node.y + 64 > 768:
            self.node.y = 768 - 64
        elif self.node.y + 64 < 0:
            ''' Under screen, player is dead '''
            self.alive = False
                