'''
Created on 10 sep 2012

@author: tomlun
'''
from lib.scene.scenemanager import SceneManager
import lib.utils
import pyglet
import pyglet.gl.gl as gl

@lib.utils.Singleton
class Root(object):
    '''
    classdocs
    '''
    
    _initialized = False
    _window = None
    _clock = None
    _frame_dispatcher = None
    _scene_managers = []
    _dt = 0
    _fps_display = None

    def __init__(self):
        '''
        Constructor
        '''
        
    def initialize(self, w_title):
        '''
        Initialize the engine and create a window
        
        @type w_title: string
        @param w_title: The title for the window
        @return: A pyglet window
        '''
        if not self._initialized:
            self._window = pyglet.window.Window(1024, 768, caption = w_title)
            self._clock = pyglet.clock.Clock()            
            self._fps_display = pyglet.clock.ClockDisplay()
            
            gl.glEnable(gl.GL_BLEND)
            gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
            
            self._initialized = True
            return self._window
        
    def render_one_frame(self):
        self._window.clear()
        self._window.dispatch_events()
        
        if self._frame_dispatcher:
            self._frame_dispatcher._frame_started(self._dt)
        
        for active_sm in [sm for sm in self._scene_managers if sm.active]:
            active_sm.render()
            
        if self._frame_dispatcher:
            self._frame_dispatcher._frame_ended(self._dt)
        
        #self._fps_display.draw()
        self._window.flip()
        self._dt = self._clock.tick()
        
    
    def create_scene_manager(self, name):
        '''
        Creates a new SceneManager
        
        @type name: string
        @param name: The name of the new scene manager
        @return: A new SceneManager
        '''
        if len([sm for sm in self._scene_managers if sm.name.lower() == name]) > 0:
            raise Exception("SceneManager with name: '%s' already exists" % name)
        
        sm = SceneManager(name)
        self._scene_managers.append(sm)
        return sm
    
    def _add_frame_dispatcher(self, dispatcher):
        """
        Adds a FrameDispatcher to the root
        
        @type dispatcher: lib.renderer.FrameDispatcher          
        """
        self._frame_dispatcher = dispatcher
    
    def _remove_scene_manager(self, sm):
        '''
        Remove a scene manager
        
        @type sm: lib.scene.SceneManager
        @param sm: The scene manager to remove  
        '''
        self._scene_managers.remove(sm)
        del sm