'''
Created on 10 sep 2012

@author: tomlun
'''
from lib.scene.scenenode import SceneNode
import lib

class SceneManager(object):
    '''
    classdocs
    '''

    def __init__(self, name):
        '''
        Constructor
        '''
        self.name = name
        self.active = True        
        self._nodes = []
        self._batches = []
        
    def render(self):
        '''
        Render the batches in the scene manager
        '''
        for batch in self._batches:
            batch.draw()
        
    def destroy(self):
        '''
        Remove everything related to the scene manager
        '''
        for node in self._nodes:
            self._nodes.remove(node)
            node.destroy()
            del node        
        del self._nodes

        for batch in self._batches:
            self._batches.remove(batch)
            del batch
        del self._batches          
                
        lib.Root()._remove_scene_manager(self)
        
    def create_scene_node(self, name):
        '''
        Creates a new scene node
        
        @type name: string
        @param name: The name of the scene node
        @return: A new scene node 
        '''
        if len([node for node in self._nodes if node.name.lower() == name]) > 0:
            raise Exception("SceneNode with name: '%s' already exists" % name)
        
        node = SceneNode(self, name)
        self._nodes.append(node)
        return node
    
    def remove_scene_node(self, node):
        '''
        Removes a node from the scene manager
        
        @type node: pyloop.scene.SceneNode
        @param node: The scene node to remove  
        '''
        self._nodes.remove(node)
        node.destroy()        
    
    def _add_batch(self, batch):
        '''
        Add a batch to the scene manager if the batch has not been added already
        
        @type batch: pyglet.graphics.Batch
        @param batch: The batch to be added  
        '''
        if not batch in self._batches:
            self._batches.append(batch)