'''
Created on 10 sep 2012

@author: tomlun
'''

class SceneNode(object):
    '''
    classdocs
    '''

    def __init__(self, sm, name):
        '''
        A new SceneNode
        
        @type sm: lib.scene.SceneManager
        @param sm: The scene manager that holds the scene node
        @type name: string
        @param name: Name of the scene node  
        '''
        self.name = name
        self.scene_manager = sm
        self.entity = None
        self._x = 0
        self._y = 0
    
    @property
    def x(self):
        return self._x
    
    @x.setter
    def x(self, value):
        self._x = value
        if self.entity:
            self.entity.x = self._x
            
    @property
    def y(self):
        return self._y
    
    @y.setter
    def y(self, value):
        self._y = value
        if self.entity:
            self.entity.y = self._y
    
    @property
    def position(self):
        return (self._x, self._y)
    
    @position.setter
    def position(self, value):
        self.x, self.y = value
    
    @property
    def right(self):
        return self.x + self.entity.width
    
    @property
    def left(self):
        return self.x
    
    @property
    def top(self):
        return self.y + self.entity.height
    
    @property
    def bottom(self):
        return self.y
        
    
    def attach(self, entity):
        '''
        Attach an entity to the scene node
        
        @type entity: lib.scene.Entity
        @param entity: The entity to attach  
        '''
        if self.entity:
            return
        
        self.entity = entity
        self.entity.x = self.x
        self.entity.y = self.y
        self.scene_manager._add_batch(entity.batch)
    
    def collides(self, node):
        if self.right < node.left or self.top < node.bottom or self.left > node.right or self.bottom > node.top:
            return False
        return True        
                   
    def destroy(self):
        if self.entity:
            self.entity.delete()